/**************************************************
opengate server
 module for Authentication by POP3S

Copyright (C) 2002 Yoshiaki Watanabe

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Email: watanaby@is.saga-u.ac.jp
**************************************************/
/*
  Thanks to programs and documentations refered.
   Sample client application cli.cpp found in the OpenSSL site 
    (developed by Sampo Kellomaki and simplified by Wade Scholine) 
   Apache module mod_auth_pam.c by Ingo Luetkebohle
*/

#include <openssl/crypto.h>
#include <openssl/x509.h>
#include <openssl/pem.h>
#include <openssl/ssl.h>
#include <openssl/err.h>

#include "opengatesrv.h"

/*************************/
/* Authenticate by POP3S */
/*************************/
int authPop3s(char *serverAddr, char *port, char *userid,
	      char *passwd)
{
  int		sockfd, n;
  char		recvline[BUFFMAXLN];
  int           authResult;
  SSL_CTX    *ctx;
  SSL        *ssl;
  SSL_METHOD *meth;

  if(serverAddr[0]=='\0'){
    err_msg("ERR in auth-pop3s: Missing address for POP3s server in config");
    return DENY;
  }

  /* POP3S server connect */
  if(port[0]=='\0'){
    sockfd = Tcp_connect(serverAddr, "pop3s");
  }else{
    sockfd = Tcp_connect(serverAddr, port);
  }

  /* prepare SSL */
  SSLeay_add_ssl_algorithms();
  meth = SSLv2_client_method();
  SSL_load_error_strings();
  ctx = SSL_CTX_new (meth);
  if( ctx == NULL ){
    err_msg("ERR in auth-pop3s: SSL_CTX_new returns NULL");
    return DENY;
  }
  
  /* ----------------------------------------------- */
  /* start SSL negotiation. */
  
  ssl = SSL_new (ctx);
  if( ssl == NULL ){
    err_msg("ERR in auth-pop3s: SSL_new returns NULL");
    return DENY;
  }

  SSL_set_fd (ssl, sockfd);
  if( SSL_connect (ssl) == -1 ){
    err_msg("ERR in auth-pop3s: SSL_connect returns error");
    return DENY;
  }

  /* --------------------------------------------------- */
  /* DATA EXCHANGE - Send a message and receive a reply. */
  /*  pop3 message exchange */

  /* get [+OK POP3 <host> <ver> server ready]*/
  if((n = readlnSSL(ssl, recvline, BUFFMAXLN)) <= 0) {
    err_msg("ERR in auth-pop3s: Pop3 server is not normal 1");
    authResult=DENY;
    goto EXITPOINT;
  }

  if(strstr(recvline,"+OK ")!=recvline){
    err_msg("ERR in auth-pop3s: Pop3 server is not normal 2");
    authResult=DENY;
    goto EXITPOINT;
  }

  /* put [user <userid>] */
  WritefmtSSL(ssl, "user %s\r\n", userid);

  /* get [+OK User name accepted, password please] */
  if((n = readlnSSL(ssl, recvline, BUFFMAXLN)) <= 0) {
    err_msg("ERR in auth-pop3s: Pop3 server is not normal 3");
    authResult=DENY;
    goto EXITPOINT;
  }

  if(strstr(recvline,"+OK ")!=recvline){
    err_msg("ERR in auth-pop3s: Pop3 server is not normal 4");
    authResult=DENY;
    goto EXITPOINT;
  }

  /* put [pass <password>] */
  WritefmtSSL(ssl, "pass %s\r\n", passwd);

  /* get [+OK Mailbox open, <count> messages] */
  if((n = readlnSSL(ssl, recvline, BUFFMAXLN)) <= 0) {
    err_msg("ERR in auth-pop3s: Pop3 server is not normal 5");
    authResult=DENY;
    goto EXITPOINT;
  }

  if(strstr(recvline,"+OK ")==recvline){
    authResult=ACCEPT;
  }else{
    authResult=DENY;
  }
  
  /* put [quit] */
  WritefmtSSL(ssl,"quit\r\n");

EXITPOINT:
  SSL_shutdown (ssl);  /* send SSL/TLS close_notify */

  /* Clean up. */
  Close(sockfd);
  SSL_free (ssl);
  SSL_CTX_free (ctx);

  return authResult;
}

int AuthPop3s(char *serverAddr, char *port, char *userid,
	      char *passwd)
{
  int ret;

  if(DEBUG) err_msg("DEBUG:=>authPop3s(%s,%s,%s,passwd)",
		    serverAddr,port,userid);
  ret=authPop3s(serverAddr,port,userid,passwd);
  if(DEBUG) err_msg("DEBUG:(%d)<=authPop3s( )",ret);

  return ret;
}
