/**************************************************
opengate server
 module for Authentication by RADIUS

Copyright (C) 2002 Yoshiaki Watanabe

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Email: watanaby@is.saga-u.ac.jp
**************************************************/
/*
  Thanks to programs and documentations refered.
   The experimental implementation by Fuyuta Sato.
   Manual files about RADIUS.
*/

#include <radlib.h>
#include "opengatesrv.h"

/*****************************************/
/* Authenticate by RADIUS */
/*****************************************/
int authRadius(char *confFile, char *userid, char *passwd)
{
  int           authResult;
  struct rad_handle *radh;
  char hostname[ADDRMAXLN];

  /* If not set, set default conf file path */
  if(confFile[0]=='\0'){
    strncpy(confFile, RADIUSCONF, ADDRMAXLN);
  }

  if( !(radh=rad_auth_open()) ){
    err_msg("ERR in auth-rad: cannot start radius");
    return DENY;
  }

  if(gethostname(hostname,ADDRMAXLN) < 0){
    err_msg("ERR in auth-rad: cannot get hostname");
    return DENY;
  }
    
  if(rad_config(radh,confFile) < 0){
    err_msg("ERR in auth-rad: cannot open conffile");
    return DENY;
  }

  if(rad_create_request(radh, RAD_ACCESS_REQUEST) < 0){
    err_msg("ERR in auth-rad: cannot create radius request");
    return DENY;
  }

  rad_put_string(radh,RAD_USER_NAME,userid);
  rad_put_string(radh,RAD_USER_PASSWORD,passwd);
  rad_put_string(radh,RAD_NAS_IDENTIFIER,hostname);
  rad_put_int(radh,RAD_SERVICE_TYPE,RAD_LOGIN);

  switch(rad_send_request(radh)){
  case RAD_ACCESS_ACCEPT:
    authResult=ACCEPT;
    break;
  case RAD_ACCESS_REJECT:
  case RAD_ACCESS_CHALLENGE:
  case RAD_ACCOUNTING_RESPONSE:
  default:
    authResult=DENY;
    break;
  }

  return authResult;
}
  
int AuthRadius(char *confFile, char *userid, char *passwd)
{
  int ret;

  if(DEBUG) err_msg("DEBUG:=>authRadius(%s,%s,passwd)",confFile,userid);
  ret=authRadius(confFile,userid,passwd);
  if(DEBUG) err_msg("DEBUG:(%d)<=authRadius( )",ret);

  return ret;
}











