/**************************************************
opengate server
 module for communication with arp 

Copyright (C) 1999-2001 Yoshiaki Watanabe

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Email: watanaby@is.saga-u.ac.jp
**************************************************/

#include "opengatesrv.h"

/*******************************************************************/
/* get MAC address for clientAddr (nnn.nnn.nnn.nnn) by arp request */
/*******************************************************************/
int getMacAddr(char *clientAddr, char* macAddr)
{
  int fd[2];
  int piperead_fd,pipewrite_fd;
  int stdin_fd=0,stdout_fd=1;
  char buf[BUFFMAXLN];
  int st;
  char *startp;
  char *endp;

  macAddr[0]='?';
  macAddr[1]='\0';

  /* create pipe */
  if(pipe(fd)!=0){
    err_msg("ERR in comm-arp: Cannot create pipe for arp\n");
    return -1;
  }
  piperead_fd=fd[0];
  pipewrite_fd=fd[1];

  /* fork */
  if(Fork()==0){
    /* child proc */

    /* connect pipeout to stdout */
    Close(stdout_fd);
    dup(pipewrite_fd);

    /* close other IO */
    Close(stdin_fd);
    Close(piperead_fd);
    Close(pipewrite_fd);

    /* exec proc */
    if(execl(ARPPATH,"arp","-n",clientAddr,(char *)0) == -1){ 
      err_quit("ERR in comm-arp: execlp error no= %d", errno);
    }
    exit(1);
  }
  /* parent proc */
  
  /* close unuse IO */
  Close(pipewrite_fd);

  /* get arp response */  
  if(Readln(piperead_fd, buf, BUFFMAXLN-1)==0){
    err_msg("ERR in comm-arp: readin error");
    return -1;
  }
  
  /* arp response takes following format           */
  /* "? (133.49.22.1) at 8:0:20:a5:4:62 [ethernet]"*/
  /* get MAC address from above string             */

  if((startp=strstr(buf," at "))==NULL) return -1;
  startp=startp+4;

  if((endp=strstr(startp, " "))==NULL) return -1;
  *endp='\0';

  strncpy(macAddr, startp, ADDRMAXLN);

  Close(piperead_fd);
    
  /* wait child end */
  wait(&st);

  return 0;
}


/****************************************/
/****************************************/
int GetMacAddr(char *clientAddr, char* macAddr)
{
  int ret;

  if(DEBUG) err_msg("DEBUG:=>getMacAddr(%s,)",clientAddr);
  ret=getMacAddr(clientAddr,macAddr);
  if(DEBUG) err_msg("DEBUG:(%d)<=getMacAddr(,%s)",ret,macAddr);

  return ret;
}  
