/**************************************************
opengate server
 module for Authentication of User

Copyright (C) 1999-2002 Yoshiaki Watanabe

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Email: watanaby@is.saga-u.ac.jp
**************************************************/

#include "opengatesrv.h"

int authFtp(char *serverAddr, char *port, char *userid, char *passwd);
int AuthFtp(char *serverAddr, char *port, char *userid, char *passwd);
int authPop3(char *serverAddr, char *port, char *userid, char *passwd);
int AuthPop3(char *serverAddr, char *port, char *userid, char *passwd);
int getAuthServer(char *serverid, char *serverAddr, char *port, char *proto, char *serviceName, char *confFile);
int GetAuthServer(char *serverid, char *serverAddr, char *port, char *proto, char *serviceName, char *confFile);
int authPam(char *serviceName, char *userid, char *passwd);
int AuthPam(char *serviceName, char *userid, char *passwd);
int authRadius(char *confFile, char *userid, char *passwd);
int AuthRadius(char *confFile, char *userid, char *passwd);
int authPop3s(char *serverAddr, char *port, char *userid, char *passwd);
int AuthPop3s(char *serverAddr, char *port, char *userid, char *passwd);

/*******************************************************/
/* Authenticate user by accessing to ftp server        */
/*  userid : user to auth                              */
/*  password : password for the user                   */
/*******************************************************/
int authenticateUser(char *userid, char *passwd)
{
  char serverAddr[ADDRMAXLN];
  char serviceName[ADDRMAXLN];
  char confFile[ADDRMAXLN];
  char port[WORDMAXLN];
  char proto[WORDMAXLN];
  char *serverid=NULL;
  char *markPnt=NULL;
  int authResult;
  char useridcpy[USERMAXLN];

  /* if no userid or no passwd, then deny */
  if(userid[0]=='\0')return DENY;
  if(passwd[0]=='\0')return DENY;

  strncpy(useridcpy, userid, USERMAXLN);

  /* separate serverid from userid */
  markPnt=strchr(useridcpy, '@');
  if(markPnt==NULL){  
    /* separator mark not found */
    serverid="default";
  }else{
    /* pick up serverid */
    serverid=markPnt+1;
    *markPnt='\0';
  }

  /* get server info from config */
  if(GetAuthServer(serverid, serverAddr, port, proto, 
		   serviceName, confFile)<0){
    return DENY;
  }

  if(strcmp(proto, "ftp")==0){
    /* authenticate by ftp access */
    authResult=AuthFtp(serverAddr, port, useridcpy, passwd);
  }else if(strcmp(proto, "pop3")==0){
    /* authenticate by pop3 access */
    authResult=AuthPop3(serverAddr, port, useridcpy, passwd);
  }else if(strcmp(proto, "pam")==0){
    /* authenticate by pam */
    authResult=AuthPam(serviceName, useridcpy, passwd);
  }else if(strcmp(proto, "radius")==0){
    /* authenticate by radius */
    authResult=AuthRadius(confFile, useridcpy, passwd);
  }else if(strcmp(proto, "pop3s")==0){
    /* authenticate by pop3s */
    authResult=AuthPop3s(serverAddr, port, useridcpy, passwd);
  }else{
    err_msg("ERR in comm-auth: Unknown protocol:%s",proto);
    return DENY;
  }

  return authResult;
}

/***********************/
/* Authenticate by FTP */
/***********************/
int authFtp(char *serverAddr, char *port, char *userid, char *passwd)
{
  int		sockfd, n;
  char		recvline[BUFFMAXLN];
  int           authResult;

  if(serverAddr[0]=='\0'){
    err_msg("ERR in comm-auth: Missing address for FTP server in config");
    return DENY;
  }

  /* FTP server connect */
  if(port[0]=='\0'){
    sockfd = Tcp_connect(serverAddr, "ftp");
  }else{
    sockfd = Tcp_connect(serverAddr, port);
  }

  /* get [220 <host> FTP server ..]*/
  if((n = readln(sockfd, recvline, BUFFMAXLN)) <= 0) {
    err_msg("ERR in comm-auth: Ftp server is not normal 1");
    Close(sockfd);
    return DENY;
  }
  if(strstr(recvline,"220")!=recvline){
    err_msg("ERR in comm-auth: Ftp server is not normal 2");
    Close(sockfd);
    return DENY;
  }

  /* put [user <userid>] */
  Writefmt(sockfd, "user %s\r\n", userid);

  /* get [331 Password required ..] */
  if((n = readln(sockfd, recvline, BUFFMAXLN)) <= 0) {
    err_msg("ERR in comm-auth: Ftp server is not normal 3");
    Close(sockfd);
    return DENY;
  }
  if(strstr(recvline,"331")!=recvline){
    err_msg("ERR in comm-auth: Ftp server is not normal 4");
    Close(sockfd);
    return DENY;
  }

  /* put [pass <password>] */
  Writefmt(sockfd, "pass %s\r\n", passwd);

  /* get [230 User <userid> logged in] */
  if((n = readln(sockfd, recvline, BUFFMAXLN)) <= 0) {
    err_msg("ERR in comm-auth: Ftp server is not normal 5");
    Close(sockfd);
    return DENY;
  }
  if(strstr(recvline,"230")==recvline){
    authResult=ACCEPT;
  }else{
    authResult=DENY;
  }
  
  /* put [quit] */
  Writefmt(sockfd,"quit\r\n");

  Close(sockfd);
  return authResult;
}

/************************/
/* Authenticate by POP3 */
/************************/
int authPop3(char *serverAddr, char *port, char *userid, char *passwd)
{
  int		sockfd, n;
  char		recvline[BUFFMAXLN];
  int           authResult;

  if(serverAddr[0]=='\0'){
    err_msg("ERR in comm-auth: Missing address for POP3 server in config");
    return DENY;
  }

  /* POP3 server connect */
  if(port[0]=='\0'){
    sockfd = Tcp_connect(serverAddr, "pop3");
  }else{
    sockfd = Tcp_connect(serverAddr, port);
  }

  /* get [+OK POP3 <host> <ver> server ready]*/
  if((n = readln(sockfd, recvline, BUFFMAXLN)) <= 0) {
    err_msg("ERR in comm-auth: Pop3 server is not normal 1");
    Close(sockfd);
    return DENY;
  }
  if(strstr(recvline,"+OK ")!=recvline){
    err_msg("ERR in comm-auth: Pop3 server is not normal 2");
    Close(sockfd);
    return DENY;
  }

  /* put [user <userid>] */
  Writefmt(sockfd, "user %s\r\n", userid);

  /* get [+OK User name accepted, password please] */
  if((n = readln(sockfd, recvline, BUFFMAXLN)) <= 0) {
    err_msg("ERR in comm-auth: Pop3 server is not normal 3");
    Close(sockfd);
    return DENY;
  }
  if(strstr(recvline,"+OK ")!=recvline){
    err_msg("ERR in comm-auth: Pop3 server is not normal 4");
    Close(sockfd);
    return DENY;
  }

  /* put [pass <password>] */
  Writefmt(sockfd, "pass %s\r\n", passwd);

  /* get [+OK Mailbox open, <count> messages] */
  if((n = readln(sockfd, recvline, BUFFMAXLN)) <= 0) {
    err_msg("ERR in comm-auth: Pop3 server is not normal 5");
    Close(sockfd);
    return DENY;
  }
  if(strstr(recvline,"+OK ")==recvline){
    authResult=ACCEPT;
  }else{
    authResult=DENY;
  }
  
  /* put [quit] */
  Writefmt(sockfd,"quit\r\n");

  Close(sockfd);
  return authResult;
}

/**********************************/
/* get auth server info from conf */
/**********************************/
int getAuthServer(char *serverid, char *serverAddr, char *port, char *proto,
		  char *serviceName, char *confFile)
{
  char *dbBuffer=NULL;
  char *dbArray[2];
  char *mstr;

  /* if no string, set default */
  if(serverid==NULL){
    serverid="default";
  }

  /* pickup server info from config file */
  dbArray[0]=CONFIGFILE;
  dbArray[1]=NULL;

  if(cgetent(&dbBuffer, dbArray, serverid)<0){
    err_msg("ERR in comm-auth: Server:%s is not found in conf", serverid);
    return -1;
  }

  /* get address */
  if(cgetstr(dbBuffer, "address",&mstr)<0){
    serverAddr[0]='\0';
  }else{
    strncpy(serverAddr, mstr, ADDRMAXLN);
    free(mstr);
  }

  /* get port */
  if(cgetstr(dbBuffer, "port", &mstr)<0){
    port[0]='\0';
  }else{
    strncpy(port, mstr, WORDMAXLN);
    free(mstr);
  }

  /* get protocol */
  if(cgetstr(dbBuffer, "protocol",&mstr)<0){
    strncpy(proto, DEFAULTPROTO, WORDMAXLN);
  }else{
    strncpy(proto, mstr, WORDMAXLN);
    free(mstr);
  }

  /* get servicename */
  if(cgetstr(dbBuffer, "servicename",&mstr)<0){
    serviceName[0]='\0';
  }else{
    strncpy(serviceName, mstr, ADDRMAXLN);
    free(mstr);
  }

  /* get conffile */
  if(cgetstr(dbBuffer, "conffile",&mstr)<0){
    confFile[0]='\0';
  }else{
    strncpy(confFile, mstr, ADDRMAXLN);
    free(mstr);
  }

  free(dbBuffer);

  return 0;
}

/**********************************/
/**********************************/

int AuthFtp(char *serverAddr, char *port, char *userid, char *passwd)
{
  int ret;

  if(DEBUG) err_msg("DEBUG:=>authFtp(%s,%s,%s,passwd)",serverAddr,port,userid);
  ret=authFtp(serverAddr,port,userid,passwd);
  if(DEBUG) err_msg("DEBUG:(%d)<=authenticateUser( )");

  return ret;
}


int AuthPop3(char *serverAddr, char *port, char *userid, char *passwd)
{
  int ret;

  if(DEBUG) err_msg("DEBUG:=>authPop3(%s,%s,%s,passwd)",serverAddr,port,userid);
  ret=authPop3(serverAddr,port,userid,passwd);
  if(DEBUG) err_msg("DEBUG:(%d)<=authPop3( )",ret);

  return ret;
}

int GetAuthServer(char *serverid, char *serverAddr, char *port, char *proto, 
		  char *serviceName, char *confFile)
{
  int ret;
  if(DEBUG) err_msg("DEBUG:=>getAuthServer(%s,,,)",serverid);
  ret=getAuthServer(serverid,serverAddr,port,proto,serviceName,confFile);
  if(DEBUG) err_msg("DEBUG:(%d)<=getAuthServer(,%s,%s,%s)",ret,serverAddr,port,proto);
  return ret;
}

int AuthenticateUser(char *userid, char *password)
{
  int ret;

  if(DEBUG) err_msg("DEBUG:=>authenticateUser(%s,passwd)",userid);
  ret=authenticateUser(userid,password);
  if(DEBUG) err_msg("DEBUG:(%d)<=authenticateUser( )",ret);

  return ret;
}
