/**************************************************
opengate server
 module for Communication through CGI 

Copyright (C) 1999-2002 Yoshiaki Watanabe

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Email: watanaby@is.saga-u.ac.jp

Programmed by Yoshiaki WATANABE
Modified by Shin-ichi TADAKI
**************************************************/

#include	"opengatesrv.h"

/* convert two-char-hex "aa" to one-number 0Xaa */ 
#define hex2num(x)  ((x)>='A' ? ((x) & 0XDF) - 'A' +10 : ((x) - '0'))

void split(char content[], char *name[], char *value[], char *next[]);
void decode(char *string);

static char language[WORDMAXLN]; /* message language in java applet */

/*******************************/
/* get the client addr         */
/*******************************/
void getClientAddr(char *clientAddr)
{
  strncpy(clientAddr, getenv("REMOTE_ADDR"), ADDRMAXLN);
}

/********************************************/
/* get userid and password from the client  */
/********************************************/
void getUserInfo(char *userid, char *password, int *durationPtr)
{
  int contentLen;
  char content[BUFFMAXLN];
  char *name[1];
  char *value[1];
  char *next[1];
  char *ptr;
  int item;
  char durationStr[WORDMAXLN];

  /* get content sent from web input */
  contentLen=atoi(getenv("CONTENT_LENGTH"));

  contentLen++; /* for terminate ch */
  if(contentLen > BUFFMAXLN) contentLen=BUFFMAXLN;
  if(fgets(content, contentLen, stdin) == NULL){
    content[0]='\0';
  }

  /* get items from string */
  userid[0]='\0';
  password[0]='\0';
  language[0]='\0';
  durationStr[0]='\0';

  ptr=content;

  for(item=0;item<4;item++){       /* get 4 items from string */
    split(ptr, name, value, next);

    if(strstr(name[0], "userid")!=NULL){
      strncpy(userid, value[0], USERMAXLN);
    }else if(strstr(name[0], "password")!=NULL){
      strncpy(password, value[0], PASSMAXLN);
    }else if(strstr(name[0], "language")!=NULL){
      strncpy(language, value[0], WORDMAXLN);
    }else if(strstr(name[0], "duration")!=NULL){
      strncpy(durationStr, value[0], WORDMAXLN);
    }
    ptr=next[0];
  }

  /* decode the values */
  decode(userid);
  decode(password);
  decode(language);
  decode(durationStr);

  /* language check */
  language[2]='\0';  /* shorten to two bytes */
  ptr=HTMLLANGS;     /* list of available languages */
  if(strstr(ptr,language)==0){ /* if not available, use first lang */
    language[0]=*ptr; language[1]=*(ptr+1); language[2]='\0';
  }
  
  /* convert duration string to interger */
  if(sscanf(durationStr,"%d",durationPtr)!=1){
    *durationPtr=DURATIONDEFAULT;
  }else{
    *durationPtr *= 60;  /* convert minutes to seconds */
  }
 if(*durationPtr <= 0){
   *durationPtr=DURATIONDEFAULT;
 }else if(*durationPtr > DURATIONMAX){
   *durationPtr=DURATIONMAX;
 }
      
 return;
}

/*********************************************/
/* put deny message to the client            */
/*********************************************/
void putClientDeny(void)
{
  FILE *fp;
  char buff[BUFFMAXLN];
  char denydoc[BUFFMAXLN];
  
  /* make path to the denydoc */
  sprintf(denydoc,"%s/%s/%s",OGPATH,language,DENYDOC);

  /* open denydoc */
  if((fp=fopen(denydoc, "r"))==NULL){
    err_msg("ERR in comm-cgi: cannot open %s", denydoc);
    PutClientMsg("Cannot find html document");
    return;
  }

  /* read html document from file and send to web */
  printf("Content-type: text/html\r\n\r\n\r\n");
  while(fgets(buff, BUFFMAXLN, fp)!=NULL){
    fputs(buff,stdout);
  }
  fputs("\r\n\r\n",stdout);
  fclose(fp);
  return;
}

/*********************************************/
/* put some message to the client            */
/*********************************************/
void putClientMsg(char *message)
{
  printf("Content-type: text/html\r\n\r\n\r\n");
  printf("<HTML><HEAD><TITLE>OpengateMsg</TITLE></HEAD> \r\n");
  printf("<BODY>         \r\n");
  printf("%s\r\n",     message);
  printf("</BODY></HTML> \r\n\r\n");
}

/*********************************************/
/* put accept message and java to the client */
/*********************************************/
void putJavaApplet(char *userid, int port, int pid)
{
  FILE *fp;
  char buff[BUFFMAXLN];
  int markfound=0;
  char acceptdoc[BUFFMAXLN];
  char *ptr;

  /* make path to acceptdoc */
  sprintf(acceptdoc,"%s/%s/%s",OGPATH,language,ACCEPTDOC);

  /* open acceptdoc */
  if((fp=fopen(acceptdoc, "r"))==NULL){
    err_msg("ERR in comm-cgi: cannot open %s", acceptdoc);
    PutClientMsg("Cannot find html document");
    return;
  }

  /* read html document from file and send to web */
  printf("Content-type: text/html\r\n\r\n\r\n");
  while(fgets(buff, BUFFMAXLN, fp)!=NULL){

    /* search insert mark and replace it */

    if(strstr(buff,APPLETMARK)!=NULL){
      /* applet insert mark found */

      markfound=1;
      printf("<applet code='Opengate.class' archive='Opengate.jar' ");     
      printf("codebase='%s' width=600 height=30 > \r\n", OPENGATEDIR );
      printf("<param name=port value='%d'>\r\n", port );
      printf("<param name=user value='%s'>\r\n", userid );
      printf("<param name=lang value='%s'>\r\n", language );
      printf("</applet>                   \r\n");

    }else if((ptr=strstr(buff,TERMINATEMARK))!=NULL){
      /* terminate URL insert mark found */
      /*  insert [http://<servaddr>:<port>/terminate<pid>] */

      *ptr='\0';  printf("%s",buff);
      printf("http://%s:%d/terminate%d", 
	     getenv("SERVER_NAME"), port, pid);
      ptr=ptr+strlen(TERMINATEMARK); printf("%s", ptr);

    }else{
      /* normal http text */
      fputs(buff,stdout);
    }
  }
  fputs("\r\n\r\n",stdout);
  fclose(fp);

  if(markfound==0){
    err_msg("ERR in comm-cgi: cannot find mark %s in %s",
                   APPLETMARK, acceptdoc);
    PutClientMsg("Cannot find applet mark");
  }
  return;
}

/************************************/
/* split value for indicated name   */
/* in content  "name=value&..."     */
/************************************/
void split(char content[], char *name[], char *value[], char *next[])
{
  char *pstr;
  
  /* set default */
  name[0]=content;
  value[0]=content+strlen(content);
  next[0]=value[0];

  /* set name end */
  if((pstr=strchr(name[0],(int)'='))==NULL) return;
  *pstr='\0';
  
  /* set value start */
  pstr++;
  value[0]=pstr;
  
  /* set value end */
  if((pstr=strchr(value[0],'&'))==NULL) return;
  *pstr='\0';

  /* set next start */
  pstr++;
  next[0]=pstr;

  return;
}

/**********************************/
/* decode text coding in web post */
/**********************************/
void decode(char *string)
{
  char *pcheck, *pinsert;

  pcheck=pinsert=string;
  while(*pcheck != '\0'){
    if(*pcheck == '+'){
      *pinsert = ' ';
    }else if(*pcheck == '%'){
      *pinsert=(char)(hex2num(*(pcheck+1))*16 + hex2num(*(pcheck+2)));
      pcheck+=2;
    }else{
      *pinsert=*pcheck;
    }
    pcheck++;
    pinsert++;
  }
  *pinsert=*pcheck;
}

/*******************************/
/*******************************/
void GetClientAddr(char *clientAddr)
{
  if(DEBUG) err_msg("DEBUG:=>getClientAddr( )");
  getClientAddr(clientAddr);
  if(DEBUG) err_msg("DEBUG:<=getClientAddr(%s)",clientAddr);
}


void GetUserInfo(char *userid, char *password, int *durationPtr)
{
  if(DEBUG) err_msg("DEBUG:=>getUserInfo( )");
  getUserInfo(userid,password,durationPtr);
  if(DEBUG) err_msg("DEBUG:<=getUserInfo(%s,passwd,%d)",userid,*durationPtr);
}

void PutJavaApplet(char *userid, int port, int pid)
{
  if(DEBUG) err_msg("DEBUG:=>putJavaApplet(%s,%d,%d)",userid,port,pid);
  putJavaApplet(userid,port,pid);
  if(DEBUG) err_msg("DEBUG:<=putJavaApplet( )");
}

void PutClientDeny(void)
{
  if(DEBUG) err_msg("DEBUG:=>putClientDeny( )");
  putClientDeny();
  if(DEBUG) err_msg("DEBUG:<=putClientDeny( )");
}

void PutClientMsg(char *message)
{
  if(DEBUG) err_msg("DEBUG:=>putClientMsg( %s )",message);
  putClientMsg(message);
  if(DEBUG) err_msg("DEBUG:<=putClientMsg( )");
}

