/*************************************************
opengate server
  module for communication with java 

Copyright (C) 1999-2001 Yoshiaki Watanabe

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Email: watanaby@is.saga-u.ac.jp
**************************************************/

#include	"opengatesrv.h"

static void intervalAlarm(int signo);
static void timeoutAlarm(int signo);
static void delayAlarm(int signo);
void getPeerAddr(int sockfd, char *peerAddr);
void GetPeerAddr(int sockfd, char *peerAddr);
void sendTerminateReply(void);
void SendTerminateReply(void);

int listenfd; /* file descriptor for listen port */
int connfd;   /* file descriptor for connection port */
int intervalAlarmRinged=0; /* interval alarm ringed */
int timeoutAlarmRinged=0;  /* timeout alarm ringed */
int delayAlarmRinged=0;    /* delay alarm ringed */

/***************************************/
/* get temp listen port of this server */
/***************************************/
int getListenPort(void)
{
  struct sockaddr_in servaddr;
  int portNo;

  listenfd=Socket(AF_INET, SOCK_STREAM, 0);

  bzero(&servaddr, sizeof(servaddr));
  servaddr.sin_family=AF_INET;
  servaddr.sin_addr.s_addr=htonl(INADDR_ANY);

  /* search unused port between PORTMIN and PORTMAX */
  for(portNo=PORTMIN; portNo<=PORTMAX; portNo++){
    servaddr.sin_port=htons(portNo);
    if(bind(listenfd, (SA *)&servaddr, sizeof(servaddr))==0)break;
  }
  if(portNo>PORTMAX)portNo=-1;  /* cannot get unused port */

  Listen(listenfd, LISTENQ);

  return portNo;
}

/************************************/
/* wait applet connection           */
/************************************/
int waitAppletConnect(char *userid, char *clientAddr, int duration, char *macAddr)
{
  socklen_t len;
  struct sockaddr_in cliaddr;  /* client IP adddress */
  char buff[BUFFMAXLN];        /* read in buffer */
  Sigfunc *sigfunc;            /* signal function save */
  int connect;                 /* java cconnect success(1) or not */
  char connectAddr[ADDRMAXLN];  /* connected client address */
  char httpStr[BUFFMAXLN];      /* HTTP GET string at terminate */
  int checkInterval=ACTIVECHECKINTERVAL; /* terminal check interval */
  char macAddrNow[ADDRMAXLN];   /* MAC address at now */
  int packetCountPrev=0;        /* packet count at previous check */
  int packetCountNow=0;         /* packet count at now */
  int noPacketPeriod=0;         /* no packet period count in check loop */
  time_t endTime;               /* the absolute time to exit */
  int leftTime;                  /* the left time length to exit */

  /* get the time to exit */
  endTime=time(NULL)+duration;

  /* loop until accepting correct user */
  len=sizeof(cliaddr);
  while(1){

    /* alarm setting */
    /* recalculate left time */
    leftTime = endTime - time(NULL);
    if(leftTime <= 0) leftTime=1;
    /* shorter time is set */
    if(leftTime < checkInterval){
      sigfunc=Signal(SIGALRM, timeoutAlarm);
      alarm(leftTime);
    }else{
      sigfunc=Signal(SIGALRM, intervalAlarm);
      alarm(checkInterval);
    }      

    /* wait connection */
    if((connfd=accept(listenfd, (SA *)&cliaddr, &len)) < 0){
      
      /* if usage timeout is reached, end the process */
      if(timeoutAlarmRinged){
	timeoutAlarmRinged=FALSE;
	err_msg("ERR in comm-java: no connection from applet");
	connect=0;
	break;
      }
      
      /* if check interval is reached, do some checking */
      if(intervalAlarmRinged){
	intervalAlarmRinged=FALSE;

	/* packet flow check */
	packetCountNow=GetPacketCount();
	if(packetCountNow==packetCountPrev){  /* no packet between checks */
	  noPacketPeriod++;
	}else{
	  noPacketPeriod=0;
	  packetCountPrev=packetCountNow;
	}
	if(noPacketPeriod*ACTIVECHECKINTERVAL >= NOPACKETINTERVAL){
	  err_msg("ERR in comm-java: no packet passed for the client");
	  connect=0;
	  break;
	}

	/* mac address check */
	if(ENABLEMACADDR){
	  GetMacAddr(clientAddr, macAddrNow);
	  if(strcmp(macAddr, macAddrNow)!=0){
	    err_msg("ERR in comm-java: mac address is changed");
	    connect=0;
	    break;
	  }
	}
	/* return to connection wait */
	continue;
      }
    }
    /* someone is connected */

    /* reset alarm */
    alarm(0);
    Signal(SIGALRM, sigfunc);

    /* is it from the correct client addr */
    GetPeerAddr(connfd, connectAddr);
    if(strcmp(connectAddr, clientAddr)!=0){
      close(connfd);
      continue;
    }
    /* set short delay alarm for read in */
    sigfunc=Signal(SIGALRM, delayAlarm);
    alarm(COMMWAITTIMEOUT);
    /* get string from connection */
    if(readln(connfd, buff, BUFFMAXLN) <=0){
      if(delayAlarmRinged)delayAlarmRinged=FALSE;
      err_msg("ERR in comm-java: cannot get userid from applet");
      connect=0;
      break;
    }
    /* some text is read */
    /* reset timeout signal */
    alarm(0);
    Signal(SIGALRM, sigfunc);
    
    /* is it the correct userid from java applet */
    if(strcmp(buff, userid)==0){
      connect=1;
      break;
    }

    /* is it the terminate request from accept.html */
    /* the request is [GET /terminate<pid> ....] */
    snprintf(httpStr, BUFFMAXLN, "GET /terminate%d", getpid());
    if(strstr(buff, httpStr)==buff){
      /* terminate request found */
      sendTerminateReply();
      connect=0;
      break;
    }

    err_msg("ERR in comm-java: unknown user or address send from applet");

    /* wait new connection */
    close(connfd);
  }
  /* reset timeout signal */
  alarm(0);
  Signal(SIGALRM, sigfunc);

  Close(listenfd);
  return connect;
}

/***************************************/
/* get addr of connected remote site   */
/***************************************/
void getPeerAddr(int sockfd, char *peerAddr)
{
  struct sockaddr *cliaddr;
  socklen_t len;
  
  cliaddr=Malloc(ADDRMAXLN);
  len=ADDRMAXLN;
  Getpeername(sockfd, cliaddr, &len);
  strncpy(peerAddr, Sock_ntop_host(cliaddr, len), ADDRMAXLN);
  free(cliaddr);

  return;
}

/***************************************************/
/* wait for close connection                       */
/***************************************************/
void waitClientClose(void)
{
  char buff[BUFFMAXLN];
  Sigfunc *defaultSigfunc;
  int savePacketCount=0;
  int helloWait=FALSE;
  int noPacketPeriod=0;
  int noReplyCount=0;

  Writefmt(connfd,"accept\r\n");

  /* set interval alarm */
  defaultSigfunc=Signal(SIGALRM, intervalAlarm);
  alarm(ACTIVECHECKINTERVAL);

  /* TCP read/write loop */
  /* this loop implement following logic */
  /*   repeat until receiving 'quit' or EOF from client         */
  /*      say 'hello' to client                                 */
  /*      receive 'hello' from client. if no reply, then quit.  */
  /*      get packet count for client after short delay.        */ 
  /*      long interval                                         */
  /*      get packet count again. if no packet, then quit.      */
 
  while(1){
    if(readln(connfd, buff, BUFFMAXLN)>0){
      /* normal read */

      /* read quit, then quit */
      if(strstr(buff,"quit") != NULL){
	break;

      /* read hello, then wait a while for packet log writing */
      }else if((helloWait)&&(strstr(buff,"hello") !=NULL)){
	helloWait=FALSE;
	noReplyCount=0;
	Signal(SIGALRM, delayAlarm);
	alarm(PACKETLOGDELAY);

      /* read any other strings, then do nothing */
      }else{
      }

    }else{
      /* abnormal read */

      /* Periodical client check */
      if(intervalAlarmRinged){
	intervalAlarmRinged=FALSE;

	/* if no packet is passed in long time, then quit */
	if(GetPacketCount()==savePacketCount){
	  noPacketPeriod++;
	  if(noPacketPeriod*ACTIVECHECKINTERVAL >= NOPACKETINTERVAL){
	    err_msg("ERR in comm-java: no packet passed for the client");
	    break;
	  }
	}else{
	  noPacketPeriod=0;
	}
	/* send hello and wait reply */
	Writefmt(connfd,"hello\r\n");
	helloWait=TRUE;
	Signal(SIGALRM, timeoutAlarm);
	alarm(COMMWAITTIMEOUT);

      /* hello reply timeout */
      }else if(timeoutAlarmRinged){
	timeoutAlarmRinged=FALSE;

	/* is the reply ignored many times */
	noReplyCount++;
	if(noReplyCount>=NOREPLYMAX){
	  /* many times, then quit */
	  err_msg("ERR in comm-java: no reply to hello");
	  break;
	}else{
	  /* not so many times, then continue */
	  helloWait=FALSE;
	  savePacketCount=GetPacketCount();
	  Signal(SIGALRM, intervalAlarm);
	  alarm(ACTIVECHECKINTERVAL);
	}	  

      /* pass the delay for firewall log renewal */
      }else if(delayAlarmRinged){
	delayAlarmRinged=FALSE;
	savePacketCount=GetPacketCount();
	Signal(SIGALRM, intervalAlarm);
	alarm(ACTIVECHECKINTERVAL);
	
      /* other abnormal read, then quit */
      }else{
	break;
      }
    }
  }
  /* reset alarm */
  Signal(SIGALRM, defaultSigfunc);
  alarm(0);
  QuitClient();
  
  return;
}


/***************************************************/
/* send reply to terminate access via connfd(not stdout) */
/***************************************************/
void sendTerminateReply(void)
{
  Writefmt(connfd,"<META HTTP-EQUIV=Pragma CONTENT=no-cache>");
  Writefmt(connfd,"<HTML><BODY> Network is closed. </BODY></HTML> \r\n\r\n");
  close(connfd);
}

/***************************************************/
/* send quit to client and close connection        */
/***************************************************/
void quitClient(void)
{
  Writefmt(connfd,"quit\r\n");
  close(connfd);
}

/***************************************************/
/* function called from interval alarm             */
/***************************************************/
static void intervalAlarm(int signo)
{
  intervalAlarmRinged=TRUE;
  return;
}

/***************************************************/
/* function called from timeout alarm              */
/***************************************************/
static void timeoutAlarm(int signo)
{
  timeoutAlarmRinged=TRUE;
  return;
}

/***************************************************/
/* function called from time delay alarm           */
/***************************************************/
static void delayAlarm(int signo)
{
  delayAlarmRinged=TRUE;
  return;
}

/***************************************************/
/***************************************************/
void GetPeerAddr(int sockfd, char *peerAddr)
{
  if(DEBUG) err_msg("DEBUG:=>getPeerAddr( )"); 
  getPeerAddr(sockfd,peerAddr);
  if(DEBUG) err_msg("DEBUG:<=getPeerAddr(,%s)",peerAddr);
}


int GetListenPort(void)
{
  int ret;

  if(DEBUG) err_msg("DEBUG:=>getListenPort( )");
  ret=getListenPort();
  if(DEBUG) err_msg("DEBUG:(%d)<=getListenPort( )",ret);

  return ret;
}

int WaitAppletConnect(char *userid, char *clientAddr, int duration, char *macAddr)
{
  int ret;

  if(DEBUG) err_msg("DEBUG:=>waitAppletConnect(%s,%s,%d,%s)",userid,clientAddr,duration,macAddr);
  ret=waitAppletConnect(userid,clientAddr,duration,macAddr);
  if(DEBUG) err_msg("DEBUG:(%d)<=waitAppletConnect( )",ret);

  return ret;
}

void WaitClientClose(void)
{
  if(DEBUG) err_msg("DEBUG:=>waitClientClose( )");
  waitClientClose();
  if(DEBUG) err_msg("DEBUG:<=waitClientClose( )");
}

void QuitClient(void)
{
  if(DEBUG) err_msg("DEBUG:=>quitClient( )");
  quitClient();
  if(DEBUG) err_msg("DEBUG:<=quitClient( )");
}

void SendTerminateReply(void)
{
  if(DEBUG) err_msg("DEBUG:=>sendTerminateReply( )");
  quitClient();
  if(DEBUG) err_msg("DEBUG:<=sendTerminateReply( )");
}
