/**************************************************
opengate server header file

Copyright (C) 1999-2001 Yoshiaki Watanabe

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Email: watanaby@is.saga-u.ac.jp
**************************************************/

#include	<sys/types.h>	/* basic system data types */
#include	<sys/socket.h>	/* basic socket definitions */
#include	<sys/time.h>	/* timeval{} for select() */
#include	<time.h>		/* timespec{} for pselect() */
#include	<netinet/in.h>	/* sockaddr_in{} and other Internet defns */
#include	<arpa/inet.h>	/* inet(3) functions */
#include	<errno.h>
#include	<fcntl.h>		/* for nonblocking */
#include	<netdb.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/stat.h>	/* for S_xxx file mode constants */
#include	<sys/uio.h>		/* for iovec{} and readv/writev */
#include	<unistd.h>
#include	<sys/wait.h>
#include	<sys/un.h>		/* for Unix domain sockets */
# include	<sys/select.h>	/* for convenience */
# include	<poll.h>		/* for convenience */
# include	<strings.h>		/* for convenience */
# include	<sys/ioctl.h>
# include	<sys/filio.h>
# include	<sys/sockio.h>
# include	<pthread.h>
#define	SA	struct sockaddr
#define	LISTENQ		1024	/* 2nd argument to listen() */
typedef	void	Sigfunc(int);	/* for signal handlers */

#include <syslog.h>
#include <ctype.h>
#include <stdarg.h>
#include <sys/signal.h>

#define DEBUG 0      /* if 1, dump many message to syslog */

#define APPLETMARK "**OpengateAppletHere**" /* applet insert point mark */
#define TERMINATEMARK "**TerminateUrlHere**" /* terminate URL insert mark */

#define DURATIONDEFAULT 1200     /* default timeout of java connect (second) */
#define DURATIONMAX 36000      /* permitted usage time without java (second) */

#define ACTIVECHECKINTERVAL 600    /* client check interval(second)   */
#define COMMWAITTIMEOUT 60        /* communication reply timeout(second) */
#define NOREPLYMAX 2               /* permitted count of no reply to hello */
#define PACKETLOGDELAY 10          /* wait log writing time(second)   */
#define NOPACKETINTERVAL 5400      /* no packet interval (second)     */
                                   /*    to judge the user has gone   */

#define LOCKTIMEOUT 10   /* ipfw excusive exec lock timeout (second)*/

#define IPFWMIN 10000    /* ipfw rule number minimum */
#define IPFWMAX 40000    /* ipfw rule number maximum */
#define IPFWINTERVAL 2   /* ipfw rule number interval */

               /* listen a port between follows */
#define PORTMIN 30000    /* minimum port number */
#define PORTMAX 60000    /* maximum port number */

#define FACILITY LOG_LOCAL1  /* syslog ID */

#define DEFAULTPROTO  "ftp"    /* default authentication protocol */

#define ADDRMAXLN 128      /* maximum address string length */
#define PASSMAXLN 128      /* maximum password string length */
#define USERMAXLN 32       /* maximum userid string length */
#define BUFFMAXLN 256      /* maximum buffer string length */
#define WORDMAXLN 16       /* maximum word length */
#define TRUE 1
#define FALSE 0
#define DENY   0
#define ACCEPT 1

#define ENABLEMACADDR 1    /* if 0, ignore MAC addr routine */ 
#define PAMSERVICENAME "opengate"    /* service name used in PAM */
#define RADIUSCONF  "/etc/radius.conf" /* default path to radius.conf */

/**********prototypes***************************************/

int authenticateUser(char *userid, char *password);
int AuthenticateUser(char *userid, char *password);

int openClientGate(char *clientAddr, char *userid, char *macAddr, char *userProperty);
int OpenClientGate(char *clientAddr, char *userid, char *macAddr, char *userProperty);

void closeClientGate(void);
void CloseClientGate(void);

int getPacketCount(void);
int GetPacketCount(void);

void getClientAddr(char *clientAddr);
void GetClientAddr(char *clientAddr);

void getUserInfo(char *userid, char *password, int *durationPtr);
void GetUserInfo(char *userid, char *password, int *durationPtr);

int getUserProperty(char *userid, char *userProperty);
int GetUserProperty(char *userid, char *userProperty);

void waitClientClose(void);
void WaitClientClose(void);

void putClientDeny(void);
void PutClientDeny(void);

void putClientMsg(char *message);
void PutClientMsg(char *message);

int getListenPort(void);
int GetListenPort(void);

void putJavaApplet(char *userid, int port, int pid);
void PutJavaApplet(char *userid, int port, int pid);

int waitAppletConnect(char *userid, char *clientAddr, int duration, char *macAddr);
int WaitAppletConnect(char *userid, char *clientAddr, int duration, char *macAddr);

ssize_t readln(int fd, void *vptr, size_t maxlen);
ssize_t Readln(int fd, void *ptr, size_t maxlen);

void Writefmt(int fd, const char *fmt, ...);

int lock(int fd);
int Lock(int fd);

int unlock(int fd);
int Unlock(int fd);

int getMacAddr(char *clientAddr, char* macAddr);
int GetMacAddr(char *clientAddr, char* macAddr);

void quitClient(void);
void QuitClient(void);

int tcp_connect(const char *host, const char *serv);
int Tcp_connect(const char *host, const char *serv);

void err_ret(const char *fmt, ...);
void err_sys(const char *fmt, ...);
void err_dump(const char *fmt, ...);
void err_msg(const char *fmt, ...);
void err_quit(const char *fmt, ...);
void errToSyslog(int i);

char * sock_ntop_host(const struct sockaddr *sa, socklen_t salen);
char * Sock_ntop_host(const struct sockaddr *sa, socklen_t salen);

int Open(const char *pathname, int oflag, mode_t mode);
void Close(int fd);
pid_t Fork(void);
void Pipe(int *fds);
void Getpeername(int fd, struct sockaddr *sa, socklen_t *salenptr);
int Socket(int family, int type, int protocol);
void Listen(int fd, int backlog);
Sigfunc * Signal(int signo, Sigfunc *func);
void * Malloc(size_t size);



