/**************************************************
opengate server
 utility routines

Copyright (C) 1999-2001 Yoshiaki Watanabe

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Email: watanaby@is.saga-u.ac.jp
**************************************************/

#include "opengatesrv.h"

/*************************************************/
/* formated write                                */
/* fd : file descriptor                          */
/* fmt : format to write                         */
/* ... : terms to write                          */
/*************************************************/
void Writefmt(int fd, const char *fmt, ...)
{
  char	buff[BUFFMAXLN];
  va_list     ap;
  int nchar, nwrt;

  va_start(ap, fmt);
  vsnprintf(buff, BUFFMAXLN, fmt, ap);	
  va_end(ap);

  nchar=strlen(buff);
  nwrt=write(fd, buff, nchar);

  return;
}


void WritefmtSSL(int fd, const char *fmt, ...)
{
  char	buff[BUFFMAXLN];
  va_list     ap;
  int nchar, nwrt;

  va_start(ap, fmt);
  vsnprintf(buff, BUFFMAXLN, fmt, ap);	
  va_end(ap);

  nchar=strlen(buff);
  nwrt=SSL_write(fd, buff, nchar);

  return;
}

/**************************************************/
/* Read one line                                  */
/*  fd: file descriptor                           */
/*  vptr: input buffer pointer                    */
/*  maxlen: buffer length                         */
/**************************************************/
ssize_t
readln(int fd, void *vptr, size_t maxlen)
{
  ssize_t	n, rc;
  char	*ptr,c;

  ptr=vptr;

  /* skip pre ctrl char */
  while(1){
    rc = read(fd, &c, 1);
    if(rc <= 0) return rc;
    if(iscntrl(c)) continue;
    break;  /* get some normal char */
  }

  /* get char loop */
  n=0;
  while(n < maxlen-1) {
    if ( rc == 1) {      /* get some char */
      if (iscntrl(c))break; /* cntrl char means line end */
      *ptr++ = c;
      n++;
    }else if (rc == 0) { /* EOF */
      break;
    } else {             /* ERR */
      if (errno != EINTR) return(-1);
    }
    rc = read(fd, &c, 1);
  }
  /* null terminate string */  
  *ptr++ = 0;
  n++;
  return(n);
}


ssize_t
readlnSSL(int fd, void *vptr, size_t maxlen)
{
  ssize_t	n, rc;
  char	*ptr,c;

  ptr=vptr;

  /* skip pre ctrl char */
  while(1){
    rc = SSL_read(fd, &c, 1);
    if(rc <= 0) return rc;
    if(iscntrl(c)) continue;
    break;  /* get some normal char */
  }

  /* get char loop */
  n=0;
  while(n < maxlen-1) {
    if ( rc == 1) {      /* get some char */
      if (iscntrl(c))break; /* cntrl char means line end */
      *ptr++ = c;
      n++;
    }else if (rc == 0) { /* EOF */
      break;
    } else {             /* ERR */
      if (errno != EINTR) return(-1);
    }
    rc = SSL_read(fd, &c, 1);
  }
  /* null terminate string */  
  *ptr++ = 0;
  n++;
  return(n);
}

/******************************/
/* lock functions using fcntl */
/******************************/
int lock(int fd)
{
  struct flock lck;
  
  lck.l_type=F_WRLCK;
  lck.l_whence=SEEK_SET;
  lck.l_start=0;
  lck.l_len=0;
  return fcntl(fd, F_SETLKW, &lck);
}
  
/********************************/
/* unlock functions using fcntl */
/********************************/
int unlock(int fd)
{
  struct flock lck;

  lck.l_type=F_UNLCK;
  lck.l_whence=SEEK_SET;
  lck.l_start=0;
  lck.l_len=0;
  return fcntl(fd, F_SETLK, &lck);
}

/****************************************/
/****************************************/

ssize_t Readln(int fd, void *ptr, size_t maxlen)
{
  ssize_t		n;

  if(DEBUG) err_msg("DEBUG:=>readln( )");
  
  if ( (n = readln(fd, ptr, maxlen)) < 0)
    err_msg("ERR in utilities: readln error");

  if(DEBUG) err_msg("DEBUG:(%d)<=readln( )",n);

  return(n);
}

int Lock(int fd)
{
  int ret;

  if(DEBUG) err_msg("DEBUG:=>lock( )");

  if((ret=lock(fd))<0){
    err_sys("lock error");
  }

  if(DEBUG) err_msg("DEBUG:(%d)<=lock( )",ret);

  return ret;
}


int Unlock(int fd)
{
  int ret;

  if(DEBUG) err_msg("DEBUG:=>unlock( )");

  if((ret=unlock(fd))<0){
    err_sys("unlock error");
  }

  if(DEBUG) err_msg("DEBUG:(%d)<=unlock( )",ret);

  return ret;
}

